//
// GameX -- CameraX Demo
// 
// Copyright (C) 2002 Rama C. Hoetzlein
// 
// This software is released under the GameX GNU GPL 
// Open Source Liscense. See the GameX documentation included
// with this source code for terms of modification, 
// distribution and re-release. 
// 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include <process.h>

#define GAMEX_MAIN
#include "gamex.hpp"

struct CubeDot {						//*** Dot-in-the-Cube Structure
	Vector3DF pos;						//		3D position of dot
	int xs, ys;							//		2D coordinates of dot on screen
	double zs;							//		Distance of dot from camera
};

										//*** Global Variables & Defines
#define MAX_DOTS		343				// Maximum number of "Dots" needed
int done, frame;						
CameraX cam;							// Camera for 3D projections
ImageX dot_img;							// TIF image of a single "Dot"
int num_dots;							// Total number of Dots in the Cube
CubeDot dot_list[MAX_DOTS];				// Array of Cube Dot structures

// Draw 3D-Cube
void drawCube ()						//*** Draw the Cube
{
	int sort[MAX_DOTS];	
	int n, j, t;
	
								// Project Dots into 2D to determine Depth from Camera
	for (n=0; n<num_dots; n++) {
		cam.Project (dot_list[n].pos, dot_list[n].xs, dot_list[n].ys, dot_list[n].zs);
		sort[n] = n;
	}	
								// Sort Dots by Depth farthest to nearest (Painter's Algorithm)
	for (n=0; n<num_dots-1; n++) 
		for (j=n; j>=0; j--) 
			if (dot_list[sort[j+1]].zs > dot_list[sort[j]].zs) {
				t = sort[j];
				sort[j] = sort[j+1];
				sort[j+1] = t;
			}
								// Draw Dots in Depth-Sorted Order (Painter's Algorithm)
	for (n=0; n<num_dots; n++) {
		j = sort[n];
		GameX.DrawMasked (dot_list[j].xs, dot_list[j].ys, &dot_img);		
	}
}

// GameX Initialize function
void initialize (void)
{
	ImageExt imgext;
	int x, y, z;
													
	GameX.Initialize (false, 800, 400);		// Initialize GameX
	
	cam.SetWindow (0, 0, 800, 400);			// Initialize Camera
	
	imgext.Load ("camdemox-dot.tif", &dot_img, DRAW_MASKED);	// Load TIF image of Dot

											// Initialize Array of Dots in Cube
	num_dots = 0;
	for (x=-12; x<=12; x+=4)
		for (y=-12; y<=12; y+=4)
			for (z=-12; z<=12; z+=4) {
				if (num_dots < MAX_DOTS) {
					dot_list[num_dots].pos.Set (x, y, z);
					num_dots++;
				}
			}
}

// GameX Main function 
void main (void)
{	
	// Set Camera Angle using current 'frame'
	cam.SetFromAngles (Vector3DF(frame, 30, 100));

	// Drawing Begins	
	GameX.DrawOpen ();	
		GameX.DrawClear ();		
		drawCube ();

	GameX.DrawClose ();
	// Drawing Ends

	frame++;		
	if (GameX.GetKey()==27) done=1;
	GameX.ViewUpdate ();
}
